import { Transaction, StateEffect } from "./transaction";
import { EditorState } from "./state";
declare type FacetConfig<Input, Output> = {
    combine?: (value: readonly Input[]) => Output;
    compare?: (a: Output, b: Output) => boolean;
    compareInput?: (a: Input, b: Input) => boolean;
    static?: boolean;
    enables?: Extension | ((self: Facet<Input, Output>) => Extension);
};
export declare class Facet<Input, Output = readonly Input[]> {
    private isStatic;
    private constructor();
    static define<Input, Output = readonly Input[]>(config?: FacetConfig<Input, Output>): Facet<Input, Output>;
    of(value: Input): Extension;
    compute(deps: readonly Slot<any>[], get: (state: EditorState) => Input): Extension;
    computeN(deps: readonly Slot<any>[], get: (state: EditorState) => readonly Input[]): Extension;
    from<T extends Input>(field: StateField<T>): Extension;
    from<T>(field: StateField<T>, get: (value: T) => Input): Extension;
}
declare type Slot<T> = Facet<any, T> | StateField<T> | "doc" | "selection";
declare const enum Provider {
    Static = 0,
    Single = 1,
    Multi = 2
}
declare class FacetProvider<Input> {
    readonly dependencies: readonly Slot<any>[];
    readonly facet: Facet<Input, any>;
    readonly type: Provider;
    readonly value: ((state: EditorState) => Input) | ((state: EditorState) => readonly Input[]) | Input;
    readonly id: number;
    extension: Extension;
    constructor(dependencies: readonly Slot<any>[], facet: Facet<Input, any>, type: Provider, value: ((state: EditorState) => Input) | ((state: EditorState) => readonly Input[]) | Input);
    dynamicSlot(addresses: {
        [id: number]: number;
    }): DynamicSlot;
}
declare type StateFieldSpec<Value> = {
    create: (state: EditorState) => Value;
    update: (value: Value, transaction: Transaction) => Value;
    compare?: (a: Value, b: Value) => boolean;
    provide?: (field: StateField<Value>) => Extension;
    toJSON?: (value: Value, state: EditorState) => any;
    fromJSON?: (json: any, state: EditorState) => Value;
};
export declare class StateField<Value> {
    private createF;
    private updateF;
    private compareF;
    private constructor();
    static define<Value>(config: StateFieldSpec<Value>): StateField<Value>;
    private create;
    init(create: (state: EditorState) => Value): Extension;
    get extension(): Extension;
}
export declare type Extension = {
    extension: Extension;
} | readonly Extension[];
export declare const Prec: {
    highest: (ext: Extension) => Extension;
    high: (ext: Extension) => Extension;
    default: (ext: Extension) => Extension;
    low: (ext: Extension) => Extension;
    lowest: (ext: Extension) => Extension;
};
export declare class Compartment {
    of(ext: Extension): Extension;
    reconfigure(content: Extension): StateEffect<unknown>;
    get(state: EditorState): Extension | undefined;
}
export declare class CompartmentInstance {
    readonly compartment: Compartment;
    readonly inner: Extension;
    constructor(compartment: Compartment, inner: Extension);
    extension: Extension;
}
export interface DynamicSlot {
    create(state: EditorState): SlotStatus;
    update(state: EditorState, tr: Transaction): SlotStatus;
    reconfigure(state: EditorState, oldState: EditorState): SlotStatus;
}
export declare class Configuration {
    readonly base: Extension;
    readonly compartments: Map<Compartment, Extension>;
    readonly dynamicSlots: DynamicSlot[];
    readonly address: {
        [id: number]: number;
    };
    readonly staticValues: readonly any[];
    readonly facets: {
        [id: number]: readonly FacetProvider<any>[];
    };
    readonly statusTemplate: SlotStatus[];
    constructor(base: Extension, compartments: Map<Compartment, Extension>, dynamicSlots: DynamicSlot[], address: {
        [id: number]: number;
    }, staticValues: readonly any[], facets: {
        [id: number]: readonly FacetProvider<any>[];
    });
    staticFacet<Output>(facet: Facet<any, Output>): any;
    static resolve(base: Extension, compartments: Map<Compartment, Extension>, oldState?: EditorState): Configuration;
}
export declare const enum SlotStatus {
    Unresolved = 0,
    Changed = 1,
    Computed = 2,
    Computing = 4
}
export declare function ensureAddr(state: EditorState, addr: number): number;
export declare function getAddr(state: EditorState, addr: number): any;
export {};
