import { Text } from "./text";
import { ChangeSet, ChangeSpec } from "./change";
import { EditorSelection, SelectionRange } from "./selection";
import { Transaction, TransactionSpec, StateEffect } from "./transaction";
import { Facet, Extension, StateField } from "./facet";
import { CharCategory } from "./charcategory";
export interface EditorStateConfig {
    doc?: string | Text;
    selection?: EditorSelection | {
        anchor: number;
        head?: number;
    };
    extensions?: Extension;
}
export declare class EditorState {
    readonly doc: Text;
    readonly selection: EditorSelection;
    private constructor();
    field<T>(field: StateField<T>): T;
    field<T>(field: StateField<T>, require: false): T | undefined;
    update(...specs: readonly TransactionSpec[]): Transaction;
    replaceSelection(text: string | Text): TransactionSpec;
    changeByRange(f: (range: SelectionRange) => {
        range: SelectionRange;
        changes?: ChangeSpec;
        effects?: StateEffect<any> | readonly StateEffect<any>[];
    }): {
        changes: ChangeSet;
        selection: EditorSelection;
        effects: readonly StateEffect<any>[];
    };
    changes(spec?: ChangeSpec): ChangeSet;
    toText(string: string): Text;
    sliceDoc(from?: number, to?: number): string;
    facet<Output>(facet: Facet<any, Output>): Output;
    toJSON(fields?: {
        [prop: string]: StateField<any>;
    }): any;
    static fromJSON(json: any, config?: EditorStateConfig, fields?: {
        [prop: string]: StateField<any>;
    }): EditorState;
    static create(config?: EditorStateConfig): EditorState;
    static allowMultipleSelections: Facet<boolean, boolean>;
    static tabSize: Facet<number, number>;
    get tabSize(): number;
    static lineSeparator: Facet<string, string | undefined>;
    get lineBreak(): string;
    static readOnly: Facet<boolean, boolean>;
    get readOnly(): boolean;
    static phrases: Facet<{
        [key: string]: string;
    }, readonly {
        [key: string]: string;
    }[]>;
    phrase(phrase: string, ...insert: any[]): string;
    static languageData: Facet<(state: EditorState, pos: number, side: 0 | 1 | -1) => readonly {
        [name: string]: any;
    }[], readonly ((state: EditorState, pos: number, side: 0 | 1 | -1) => readonly {
        [name: string]: any;
    }[])[]>;
    languageDataAt<T>(name: string, pos: number, side?: -1 | 0 | 1): readonly T[];
    charCategorizer(at: number): (char: string) => CharCategory;
    wordAt(pos: number): SelectionRange | null;
    static changeFilter: Facet<(tr: Transaction) => boolean | readonly number[], readonly ((tr: Transaction) => boolean | readonly number[])[]>;
    static transactionFilter: Facet<(tr: Transaction) => TransactionSpec | readonly TransactionSpec[], readonly ((tr: Transaction) => TransactionSpec | readonly TransactionSpec[])[]>;
    static transactionExtender: Facet<(tr: Transaction) => Pick<TransactionSpec, "effects" | "annotations"> | null, readonly ((tr: Transaction) => Pick<TransactionSpec, "effects" | "annotations"> | null)[]>;
}
