import { ChangeSet, ChangeDesc, ChangeSpec } from "./change";
import { EditorState } from "./state";
import { EditorSelection } from "./selection";
import { Extension } from "./facet";
import { Text } from "./text";
export declare class Annotation<T> {
    readonly type: AnnotationType<T>;
    readonly value: T;
    static define<T>(): AnnotationType<T>;
    private _isAnnotation;
}
export declare class AnnotationType<T> {
    of(value: T): Annotation<T>;
}
interface StateEffectSpec<Value> {
    map?: (value: Value, mapping: ChangeDesc) => Value | undefined;
}
export declare class StateEffectType<Value> {
    readonly map: (value: any, mapping: ChangeDesc) => any | undefined;
    of(value: Value): StateEffect<Value>;
}
export declare class StateEffect<Value> {
    readonly value: Value;
    map(mapping: ChangeDesc): StateEffect<Value> | undefined;
    is<T>(type: StateEffectType<T>): this is StateEffect<T>;
    static define<Value = null>(spec?: StateEffectSpec<Value>): StateEffectType<Value>;
    static mapEffects(effects: readonly StateEffect<any>[], mapping: ChangeDesc): readonly StateEffect<any>[];
    static reconfigure: StateEffectType<Extension>;
    static appendConfig: StateEffectType<Extension>;
}
export interface TransactionSpec {
    changes?: ChangeSpec;
    selection?: EditorSelection | {
        anchor: number;
        head?: number;
    };
    effects?: StateEffect<any> | readonly StateEffect<any>[];
    annotations?: Annotation<any> | readonly Annotation<any>[];
    userEvent?: string;
    scrollIntoView?: boolean;
    filter?: boolean;
    sequential?: boolean;
}
export declare class Transaction {
    readonly startState: EditorState;
    readonly changes: ChangeSet;
    readonly selection: EditorSelection | undefined;
    readonly effects: readonly StateEffect<any>[];
    readonly scrollIntoView: boolean;
    private constructor();
    get newDoc(): Text;
    get newSelection(): EditorSelection;
    get state(): EditorState;
    annotation<T>(type: AnnotationType<T>): T | undefined;
    get docChanged(): boolean;
    get reconfigured(): boolean;
    isUserEvent(event: string): boolean;
    static time: AnnotationType<number>;
    static userEvent: AnnotationType<string>;
    static addToHistory: AnnotationType<boolean>;
    static remote: AnnotationType<boolean>;
}
export declare function resolveTransaction(state: EditorState, specs: readonly TransactionSpec[], filter: boolean): Transaction;
export declare function asArray<T>(value: undefined | T | readonly T[]): readonly T[];
export {};
